#import "ImageView.h"
#import "Utility.h"


@implementation ImageView


-(void)awakeFromNib
{
}

-(void)dealloc
{
	[mImageURL release];
	[super dealloc];
}

-(void)setImageWithURL:(NSURL *)url
{
	// backup the image url
	if (url != nil && mImageURL != url)
	{
		[mImageURL release];
		mImageURL = [url copy];

        // checks if a layer is already set
        if ([self overlayForType:IKOverlayTypeImage] == nil)
            [self setOverlay:[CALayer layer] forType:IKOverlayTypeImage];

		// clear the overlay animation (if it was previously set by another gif)
		[[self overlayForType:IKOverlayTypeImage] removeAllAnimations];
		
		// check if the image a gif
		if ([Utility isGIF:[url path]] == YES)
		{
			// create the image
			NSImage * image = [[NSImage alloc] initWithContentsOfFile:[url path]];
			
			// get the image representations
			NSArray * reps = [image representations];
			for (NSImageRep * rep in reps)
			{
				// find the bitmap representation
				if ([rep isKindOfClass:[NSBitmapImageRep class]] == NO)
					continue;

				// get the bitmap representation
				NSBitmapImageRep * bitmapRep = (NSBitmapImageRep *)rep;
				
				// check multiframe gif
				int numFrame = [[bitmapRep valueForProperty:NSImageFrameCount] intValue];
				if (numFrame == 0)
					break;
				
				// create the layer, animation and value array
				CAKeyframeAnimation * animation = [CAKeyframeAnimation animationWithKeyPath:@"contents"];
				NSMutableArray * values = [NSMutableArray array];
				
				// loop through the frames
				float animationDuration = 0.0f;
				for (int i = 0; i < numFrame; ++i)
				{
					// set the current frame
					[bitmapRep setProperty:NSImageCurrentFrame withValue:[NSNumber numberWithInt:i]];
					
					// if the frame duration is 0, replace it by a default of 1/24 second
					if ([[bitmapRep valueForProperty:NSImageCurrentFrameDuration] floatValue] <= 0.000001f)
						[bitmapRep setProperty:NSImageCurrentFrameDuration withValue:[NSNumber numberWithFloat:1.0f / 20.0f]];
					
					// add the CGImageRef to this frame to the value array
					[values addObject:(id)[bitmapRep CGImage]];
					
					// update the duration of the animation
					animationDuration += [[bitmapRep valueForProperty:NSImageCurrentFrameDuration] floatValue];
				}
				
				// setup the animation
				[animation setValues:values];
				[animation setCalculationMode:@"discrete"];
				[animation setDuration:animationDuration];
				[animation setRepeatCount:HUGE_VAL];

				// add the animation to the layer
				[[self overlayForType:IKOverlayTypeImage] addAnimation:animation forKey:@"contents"];
				
				// stops at the first valid representation
				break;
			}
			
			// release the image
			[image release];
		}
	}

	[super setImageWithURL:url];
}

-(void)keyDown:(NSEvent *)theEvent
{
	// autoresizes might be set to NO when zooming, so we
	// restore it to YES here (changing the current image
	// restores it)
	[self setAutoresizes:YES];

	// little hack : for some reasons, when we
	// go into fullscreen mode, the firstResponder stops getting
	// the events. So we override the keyDown method of the image view
	// and simply forward them to the first responder.
	[[mWindow firstResponder] keyDown:theEvent];
}

-(void)toggleFullscreen
{
	if ([self isInFullScreenMode] == YES)
	{
		// exit fullscreen
		[self exitFullScreenModeWithOptions:nil];
		[NSCursor unhide];
	}
	else
	{
		// create a dictionary for the fullscreen options
		// NSFullScreenModeWindowLevel is used to display on top of everything
		// NSApplicationPresentationDefault is used so that we can still use Spaces, swtich process, etc.
		NSMutableDictionary * options = [NSMutableDictionary dictionary];
		[options setValue:[NSNumber numberWithInt:NSMainMenuWindowLevel + 1]
				   forKey:NSFullScreenModeWindowLevel];
		[options setValue:[NSNumber numberWithInt:NSApplicationPresentationDefault]
				   forKey:NSFullScreenModeApplicationPresentationOptions];

		// enter fullscreen mode and make the view the first responder
		[self enterFullScreenMode:[NSScreen mainScreen] withOptions:options];
		[NSCursor hide];

		// reload the image to have a bigger version than the preview
		[self reloadImage];
	}
}

-(void)magnifyWithEvent:(NSEvent *)event
{
	float newZoomFactor = [self zoomFactor] * ([event magnification] + 1.0f);
	if (newZoomFactor < 3.0f && newZoomFactor > 0.05f)
		[self setZoomFactor:newZoomFactor];
}

-(void)reloadImage
{
	[self setImageWithURL:nil];
	[self setImageWithURL:mImageURL];
}

@end
